#!/bin/bash
# smclt: Samsung Magican Command Line Tool
#  
# version: 1-r1.1.1
# shows Samsung S.M.A.R.T. information human readable
# supported SSDs: Samsung SATA, mSATA and M.2 SATA SSDs
# 
# based on samsung_ssd_get_lifetime_writes.bash by j. d .g. leaver
# blog: http://www.jdgleaver.co.uk/blog/2014/05/23/samsung_ssds_reading_total_bytes_written_under_linux.html
# source code: http://www.jdgleaver.co.uk/blog/files/samsung_ssd_get_lifetime_writes.bash
# modifed by Fred Bogus Trumper 2023
# Copyright (c) 2023 Fred Bogus Trumper
# 
# license: GNU GENERAL PUBLIC LICENSE, Version 3, 29 June 2007
# license text: https://www.gnu.org/licenses/gpl-3.0.txt
# support forum: https://www.newnigma2.to
# 
# release date: 2023-02-26
# release notes:
#       - replaced bc by awk in code for DreamOS (bc does not round correctly either)
#       - added samsung sata ssd database
#         supported devices (database): Samsung 840, 750, 850, 860, 870 series
#       - show the value of MTBF and TBW of the device
#       - show firmware version
#       - show temperature
#       - device list
#       - find Samsung SSD device
#       - select different devices
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
# 


DATABASE="/usr/lib/enigma2/python/Plugins/SystemPlugins/SamsungMagican/database/smclt.db"
DB_MODEL=
LINE="----------------------------------------"
LINE_LENGH="40"
BORDER="1"
LINE=$(for N in $(seq 1 $LINE_LENGH);do echo -n -;done;echo)
BLOCK_LENGH=
UNIT=
DEVICE=
MODEL_FAMILY_TAG="Model Family:"
MODEL_TAG="Device Model:"
DRIVE_TAG="Rotation Rate:"
ON_TIME_TAG="Power_On_Hours"
WEAR_COUNT_TAG="Wear_Leveling_Count"
LBAS_WRITTEN_TAG="Total_LBAs_Written"
LBA_SIZE_TAG="Sector Size:"
TEMPERATURE_TAG="Temperature"
FIRMWARE_TAG="Firmware Version:"
SMART_INFO=
MODEL_FAMILY=
MODEL=
FIRMWARE=
TEMPERATURE=
LBA_SIZE=
LBA_SIZE_TEXT=
ON_TIME=
WEAR_COUNT=
LBAS_WRITTEN=
MB_WRITTEN=
GB_WRITTEN=
TB_WRITTEN=
PB_WRITTEN=
MEAN_WRITE_RATE=
MTBF=
TBW=
BYTES_PER_MB=1048576
BYTES_PER_GB=1073741824
BYTES_PER_TB=1099511627776
BYTES_PER_PB=1125899906842624
HOSTNAME_STRING=
DEVICE_STRING=
MODEL_STRING=
FIRMWARE_STRING=
MTBF_STRING=
TBW_STRING=
DRIVE_HEALTH_STRING=
TEMPERATURE_STRING=
POWER_ON_STRING=
LBA_STRING=
MB_WRITTEN_STRING=
GB_WRITTEN_STRING=
TB_PERCENT=
TB_WRITTEN_STRING=
PB_PERCENT=
PB_WRITTEN_STRING=
MEAN_WRITE_RATE_STRING=
get_smart_infos (){
SMART_INFO=$(/usr/sbin/smartctl -a "$DEVICE")
}
get_block (){
if echo $1|grep "%" &>/dev/null;then FIRST_LENGH=$(( ${#1}-1 ));else read FIRST_LENGH <<< ${#1};fi
if echo $2|grep "%" &>/dev/null;then SECOND_LENGH=$(( ${#2}-1 ));else read SECOND_LENGH <<< ${#2};fi
if echo $3|grep "%" &>/dev/null;then THIRD_LENGH=$(( ${#3}-1 ));else read THIRD_LENGH <<< ${#3};fi
if echo $4|grep "%" &>/dev/null;then FORTH_LENGH=$(( ${#4}-1 ));else read FOURTH_LENGH <<< ${#4};fi
BLOCK_LENGH=$(( $LINE_LENGH-($BORDER*2)-$FIRST_LENGH-$SECOND_LENGH-$THIRD_LENGH-$FOURTH_LENGH ))
printf " $1%"$BLOCK_LENGH"s$2$3$4\n"
}
samsung_magican (){
get_smart_infos
MODEL_FAMILY=$(echo "$SMART_INFO" | grep "$MODEL_FAMILY_TAG" | awk -F \: '{print $2}' | sed 's/ *$//g')
MODEL=$(echo "$SMART_INFO" | grep "$MODEL_TAG" | awk -F \: '{print $2}' | sed -e 's/^[ \t]*//;s/[ \t]*$//')
FIRMWARE=$(echo "$SMART_INFO" | grep "$FIRMWARE_TAG" | awk '{print $3}')
TEMPERATURE=$(echo "$SMART_INFO" | grep "$TEMPERATURE_TAG" | awk '{print $10}')
LBA_SIZE=$(echo "$SMART_INFO" | grep "$LBA_SIZE_TAG" | awk '{print $3}')
LBA_SIZE_TEXT=$(echo "$SMART_INFO" | grep "$LBA_SIZE_TAG" | awk '{print $3, $4}')
ON_TIME=$(echo "$SMART_INFO" | grep "$ON_TIME_TAG" | awk '{print $10}')
WEAR_COUNT=$(echo "$SMART_INFO" | grep "$WEAR_COUNT_TAG" | awk '{print $4}' | sed -e 's/^[ \t]*0//;s/[ \t]*$//')
LBAS_WRITTEN=$(echo "$SMART_INFO" | grep "$LBAS_WRITTEN_TAG" | awk '{print $10}')
BYTES_WRITTEN=$(echo $(( $LBAS_WRITTEN * $LBA_SIZE )) )
MB_WRITTEN=$(awk 'BEGIN {printf("%.3f\n", "'"$BYTES_WRITTEN"'" / "'"$BYTES_PER_MB"'" )}')
GB_WRITTEN=$(awk 'BEGIN {printf("%.3f\n", "'"$BYTES_WRITTEN"'" / "'"$BYTES_PER_GB"'" )}')
TB_WRITTEN=$(awk 'BEGIN {printf("%.3f\n", "'"$BYTES_WRITTEN"'" / "'"$BYTES_PER_TB"'" )}')
PB_WRITTEN=$(awk 'BEGIN {printf("%.3f\n", "'"$BYTES_WRITTEN"'" / "'"$BYTES_PER_PB"'" )}')
DB_MODEL=$(sed 's/ mSATA//g;s/ M.2//g' <<< $MODEL)
if grep "$DB_MODEL" $DATABASE |grep -v "^#" &>/dev/null;then
TBW=$(grep "$DB_MODEL" $DATABASE|awk -F\; '{print $2}'| sed -e 's/^[ \t]*//;s/[ \t]*$//')
MTBF=$(grep "$DB_MODEL" $DATABASE|awk -F\; '{print $3}'| sed -e 's/^[ \t]*//;s/[ \t]*$/ hrs/')
UNIT=$(sed 's/[0-9]//g;s/\.//g' <<< $TBW)
TBW=$(sed 's/[a-z]//g;s/[A-Z]//g' <<< $TBW)
else
MTBF="unknown"
TBW="unknown"
fi
echo $LINE
printf "%6sSamsung SATA SSD information\n"
echo $LINE
MODEL_STRING=$(get_block "Model:" "$MODEL")
echo "$MODEL_STRING"
DEVICE_STRING=$(get_block "Device Name:" "$DEVICE")
echo "$DEVICE_STRING"
FIRMWARE_STRING=$(get_block "Firmware Version:" "$FIRMWARE")
echo "$FIRMWARE_STRING"
SECTOR_SIZE_STRING=$(get_block "Sector Size:" "$LBA_SIZE_TEXT")
echo "$SECTOR_SIZE_STRING"
MTBF_STRING=$(get_block "MTBF:" "$MTBF")
echo "$MTBF_STRING"
TBW_STRING=$(get_block "TBW:" "$TBW$UNIT")
echo "$TBW_STRING"|tr "." ","
echo $LINE
if [  -z "$WEAR_COUNT" -a -z "$MEDIA_WEAR_OUT" ];then
WEAR_COUNT="unknown"
fi
if [ -n "$MEDIA_WEAR_OUT" ];then
read WEAR_COUNT <<< "$MEDIA_WEAR_OUT"
echo  $MEDIA_WEAR_OUT $WEAR_COUNT
fi
if [ "$WEAR_COUNT" == "unknown" ];then
DRIVE_HEALTH_STRING=$(get_block "Drive health:" "$WEAR_COUNT")
else
DRIVE_HEALTH_STRING=$(get_block "Drive health:" "$WEAR_COUNT%%")
fi
echo "$DRIVE_HEALTH_STRING"
TEMPERATURE_STRING=$(get_block "Temperature:" "${TEMPERATURE}YZ"|sed 's/YZ/°C/')
echo "$TEMPERATURE_STRING"
POWER_ON_STRING=$(get_block "Power on time:" "$(echo $ON_TIME | sed ':a;s/\B[0-9]\{3\}\>/,&/;ta') hrs")
echo "$POWER_ON_STRING"
echo $LINE
MB_WRITTEN_STRING=$(get_block "Data written:" "$(echo $MB_WRITTEN | tr "." "," | sed ':a;s/\B[0-9]\{3\}\>/.&/;ta') MB")
echo "$MB_WRITTEN_STRING"
GB_WRITTEN_STRING=$(get_block "" "$(echo $GB_WRITTEN | tr "." "," | sed ':a;s/\B[0-9]\{3\}\>/.&/;ta') GB")
echo "$GB_WRITTEN_STRING"
if [ "$UNIT" == "TB" ];then
TB_PERCENT=$(awk 'BEGIN {printf("%.2f\n", ("'"$TB_WRITTEN"'"*100/"'"$TBW"'"))}')
TB_WRITTEN_STRING=$(get_block "$TB_PERCENT%% of $TBW$UNIT" "$(echo $TB_WRITTEN | tr "." "," | sed ':a;s/\B[0-9]\{3\}\>/.&/;ta')"  " TB")  ##  "($TB_PERCENT%%)")
else
TB_WRITTEN_STRING=$(get_block "" "$(echo $TB_WRITTEN | tr "." "," | sed ':a;s/\B[0-9]\{3\}\>/.&/;ta') TB")
fi
echo "$TB_WRITTEN_STRING"|tr "." ","
if [ "$UNIT" == "PB" ];then
PB_PERCENT=$(awk 'BEGIN {printf("%.2f\n", ("'"$PB_WRITTEN"'"*100/"'"$TBW"'"))}')
PB_WRITTEN_STRING=$(get_block "$PB_PERCENT%% of $TBW$UNIT" "$(echo $PB_WRITTEN | tr "." "," | sed ':a;s/\B[0-9]\{3\}\>/.&/;ta')"  " PB")  ##  "($TB_PERCENT%%)")
echo "$PB_WRITTEN_STRING"|tr "." ","
fi
echo
MEAN_WRITE_RATE=$(get_block $(awk 'BEGIN {printf("%.2f\n", "'"$MB_WRITTEN"'" / "'"$ON_TIME"'" )}'))
MEAN_WRITE_RATE_STRING=$(get_block "Mean write rate:"  "$(echo $MEAN_WRITE_RATE | tr "." "," | sed ':a;s/\B[0-9]\{3\}\>/.&/;ta') MB/hr")
echo "$MEAN_WRITE_RATE_STRING"
echo -e "$LINE\n"
}
get_default_storage (){
DEVICE=$(ls -l /dev/disk/by-uuid/$(grep default_device /etc/enigma2/settings|awk -F\= '{print $2}')|awk -F\/..\/ '{print "/dev/"$2}'|cut -b 1-8)
}
list_scsi_devices (){
echo -e "attached SCSI devices (USB and SATA):\n"
lsblk -Sp -o NAME,TYPE,VENDOR,MODEL,TRAN
echo
exit 1
}
list_smart_devices (){
echo -e "attached SCSI devices with S.M.A.R.T. support:\n"
for line in $(smartctl --scan|awk '{print $1}')
do 
#echo "$line: $(smartctl -a $line|grep Model|grep -v "Family"|cut -d ":" -f2)"
echo -e "$line:\t$(smartctl -a $line|grep Model|grep -v "Family"|awk -F \: '{print $2}' | sed 's/^[ \t]*//g')"
done
echo
exit 2
}
list_magican_devices (){
echo -e "attached Samsung SATA SSDs with S.M.A.R.T. support
also M.2 and mSATA SSDs in SATA adapters/cases\n"
for DEVICE in $(smartctl --scan|awk '{print $1}')
do
SMART_INFO=$(/usr/sbin/smartctl -a "$DEVICE")
DRIVE=$(echo "$SMART_INFO" | grep "$DRIVE_TAG" | awk -F \: '{print $2}' | sed 's/^[ \t]*//g')
MODEL=$(echo "$SMART_INFO" | grep "$MODEL_TAG" | awk -F \: '{print $2}' | sed 's/^[ \t]*//g')
if grep -i "samsung" <<< $MODEL &>/dev/null;then
if [ "$DRIVE" == "Solid State Device" ]
then
echo -e "$DEVICE:\t$MODEL"
fi
fi	
done
echo
exit 3
}
short_help (){
echo -e "usage: [<device>|option]
\toptions: -e2,-la,-ls,-lm,-V,-h\n"
exit 4
}
show_help (){
echo -e "smclt
usage: [<device>|option]\n
   devices:
   -e2\t\t\tenigma2 default storage device
   <device>\t\te.g. /dev/sda\n
   options:
   -la, --list-all\tlist all attached SCSI devices
   -ls, --list-smart\tlist all attached SCSI devices
\t\t\twith S.M.A.R.T. support
   -lm, --list-magican\tlist all attached Samsung SATA SSDs
\t\t\twith S.M.A.R.T. support
   -V, --version\tinformation about this tool
   -h, --help\t\tshow this help
"
exit 5
}
case $1 in
-e2)
DEVICE=$(ls -l /dev/disk/by-uuid/$(grep default_device /etc/enigma2/settings|awk -F\= '{print $2}')|awk -F\/..\/ '{print "/dev/"$2}'|cut -b 1-8)
#echo "show magican - Enigma2 default storage device: $DEVICE"
;;
-la|--list-all)         list_scsi_devices;;
-ls|--list-smart)       list_smart_devices;;
-lm|--list-magician)    list_magican_devices;;
-V|--version)           sed -n '2,37p' $0 | sed 's/# //g';exit 10;;
-h|--help|help)         show_help;exit 1;;
"")                     short_help;;
esac
if [ "$1" != "-e2" ];then
DEVICE=$(echo $1)
fi
if [ ! -b $DEVICE ];then
echo -e "$DEVICE not found or not a SCSI device!\ntry option -la or -h\n"
exit 6	
elif ! grep "SMART support is: Enabled" <<< "$(smartctl -i $DEVICE)" &>/dev/null;then 
echo -e "$DEVICE: S.M.A.R.T. support not enabled or not available\ntry option -ls or -h\n"
exit 7
elif ! grep "$DRIVE_TAG" <<< "$(smartctl -a $DEVICE)"|grep "Solid State Device" &>/dev/null;then
echo -e "$DEVICE: no Solid State Drive\ntry option -ls or -h\n"
exit 8
elif ! grep "$MODEL_TAG" <<< "$(smartctl -a $DEVICE)"|grep -i "samsung" &>/dev/null;then
echo -e "$DEVICE: $(smartctl -i $DEVICE|grep "$MODEL_TAG" | awk -F \: '{print $2}' | sed 's/^[ \t]*//g') not supported\ntry option -lm or -h\n"
exit 9
else
samsung_magican
fi

exit 0
